;; common procedures for split and join; currently used as an include

(module xattr-common
 (parse-record:rsync-list-only
  list-format
  ;; attribute filename transforms
  xattr-filename:individual-prefix
  xattr-filename:individual-suffix
  xattr-filename:dedicated-prefix
  )

(import scheme chicken)
(use regex) (import irregex)
(use matchable)
(import (only extras read-line))
(import (only data-structures conc))
(require-library srfi-13)

;; Unescape \#ddd octal escapes; ignores illegal values.
(define unescape-octals
  (let ((rx (irregex '(: "\\#" (submatch (= 3 num)))))
        (octal->char (lambda (m)
                       (let* ((num (string->number
                                    (irregex-match-substring m 1) 8)))
                         (if num
                             (string (integer->char num))
                             (irregex-match-substring m 0))))))
    (lambda (str)
      (irregex-replace/all rx str octal->char))))

;; -rw-r--r--          39 2007/03/25 20:09:10 .gtk-bookmarks
;; lrw-r--r--          39 2007/03/25 20:09:10 .gtk-bookmarks -> /dev/null
(define parse-record:rsync-list-only
  (let ((rx (irregex '(: bos
                         (submatch ascii)                  ; type
                         (= 4 (: (+ (~ space)) (+ space))) ; skip 4 fields
                         (submatch (+ any)))))) ; filename (rest)
    (lambda (record)
      (match (string-search rx record)
             ((rec type filename)
              (let ((filename (unescape-octals filename)))
                (cond ((string=? type "l")
                       (match (string-split-fields " -> " filename #:infix)
                              ((src) src)
                              ((src dst) src)
                              (else
                               (warning 'parse-record:rsync-list-only
                                        (conc "multiple -> in symlink, skipping: " filename))
                               #f)))
                      (else filename))))
             (#f #f)))))

(define parse-record:unified
  (lambda (record)
    (match record
           (('file filename . rest)
            record)
           (else
            (error "invalid input record" record)
            #f))))

;; Chicken doesn't have a native way to read until delimiter,
;; so this will be noticeably slower than read-line.
(define read-nul-delimited
  (lambda (port)
    (let loop ((c (read-char)) (L '()))
      (cond ((eof-object? c)
             (if (null? L)
                 c
                 (reverse-list->string L)))
            ((char=? c #\nul)
             (reverse-list->string L))
            (else
             (loop (read-char) (cons c L)))))))

(define (list-format format)  ; (format-name read-record parse-record)
  (assq format
        `((rsync-list-only ,read-line ,parse-record:rsync-list-only)
          (find ,read-line ,(lambda (line) line))
          (find0 ,read-nul-delimited ,(lambda (line) line))
          (unified ,read ,parse-record:unified))))

;;; attribute filename transforms

(import (only files pathname-strip-directory pathname-directory make-pathname))

;; Directories are in parent (like files) -- e.g. /home/jim/tmp -> /home/jim/.@_tmp,
;; not /home/jim/tmp/.@_. as in xbup.  Ensure that . is included in the root directory
;; so that root dir attribute is stored in .@_. ; rsync does this when backing up
;; xxx/ (or even /).  When backing up xxx, the parent is always inside the backup
;; root.

;; #;55> (xattr-filename:individual-prefix ".")
;; ".@_."
;; (xattr-filename:individual-prefix "/Users/jim")
;; "/Users/.@_jim"
(define (xattr-filename:individual-prefix fn base)
  (make-pathname (or base "")
                 (make-pathname
                  (pathname-directory fn)
                  (string-append ".@_" (pathname-strip-directory fn)))))
(define (xattr-filename:individual-suffix fn base)
  (make-pathname (or base "")
                 (string-append fn "._@")))

;; #;45> (xattr-filename:dedicated-prefix "/")
;; "/.@@/.@_"
;; #;45> (xattr-filename:dedicated-prefix "/Users")
;; "/.@@/.@_Users"
(define (xattr-filename:dedicated-prefix fn base)
  (make-pathname (or base "")
                 (make-pathname
                  (make-pathname (pathname-directory fn) ".@@")
                  (string-append ".@_" (pathname-strip-directory fn))))))

;; Same as individual-prefix, replacing .@_ with ._
;; NB: OS X expects attribute for directory A in parent directory
;;     as ._A -- which is generally the case for us, but if you
;;     rsync ~/tmp/, then ~/tmp/._. gets tmp's attributes.
;;     In this case, you must rsync ~/tmp to get attributes in ~/._tmp .
;;     OS X doesn't seem to allow attributes in root directories.
;;     FixupResourceForks accepts ._. though.
(define (xattr-filename:appledouble fn base)
  (make-pathname (or base "")
                 (make-pathname
                  (pathname-directory fn)
                  (string-append "._" (pathname-strip-directory fn)))))

